@@ -1,4 +1,5 @@
-__version__ = '0.2.2'
+"""Automated rejection and repair of epochs in M/EEG."""
+__version__ = '0.3'
 
 from .autoreject import _GlobalAutoReject, _AutoReject, AutoReject
 from .autoreject import RejectLog, read_auto_reject
