@@ -20,7 +20,7 @@ from mne.viz import plot_epochs as plot_mne_epochs
 from sklearn.base import BaseEstimator
 from sklearn.model_selection import RandomizedSearchCV
 from sklearn.model_selection import KFold, StratifiedShuffleSplit
-from sklearn.model_selection import cross_val_score
+from sklearn.model_selection import cross_val_score, check_cv
 
 from .utils import (_clean_by_interp, interpolate_bads, _get_epochs_type,
                     _pbar, _handle_picks, _check_data, _compute_dots,
@@ -50,7 +50,7 @@ def validation_curve(epochs, y=None, param_name="thresh", param_range=None,
     ----------
     epochs : instance of mne.Epochs.
         The epochs.
-    y : array | None
+    y : array | None
         The labels.
     param_name : str
         Name of the parameter that will be varied.
@@ -185,10 +185,11 @@ def get_rejection_threshold(epochs, decim=1, random_state=None,
     ch_types : str | list of str | None
         The channel types for which to find the rejection dictionary.
         e.g., ['mag', 'grad']. If None, the rejection dictionary
-        will have keys ['mag', 'grad', 'eeg', 'eog'].
-    cv : int
-        The number of folds used. Defaults to 5.
-    verbose : bool
+        will have keys ['mag', 'grad', 'eeg', 'eog', 'hbo', 'hbr'].
+    cv : a scikit-learn cross-validation object
+        Defaults to cv=5
+    verbose : boolean
+        The verbosity of progress messages.
         If False, suppress all output messages.
 
     Returns
@@ -196,8 +197,8 @@ def get_rejection_threshold(epochs, decim=1, random_state=None,
     reject : dict
         The rejection dictionary with keys as specified by ch_types.
 
-    Note
-    ----
+    Notes
+    -----
     Sensors marked as bad by user will be excluded when estimating the
     rejection dictionary.
     """
@@ -208,7 +209,7 @@ def get_rejection_threshold(epochs, decim=1, random_state=None,
                          'or str. Got %s' % type(ch_types))
 
     if ch_types is None:
-        ch_types = ['mag', 'grad', 'eeg', 'eog']
+        ch_types = ['mag', 'grad', 'eeg', 'eog', 'hbo', 'hbr']
     elif isinstance(ch_types, str):
         ch_types = [ch_types]
 
@@ -216,7 +217,8 @@ def get_rejection_threshold(epochs, decim=1, random_state=None,
         epochs = epochs.copy()
         epochs.decimate(decim=decim)
 
-    cv = KFold(n_splits=cv, random_state=random_state)
+    cv = check_cv(cv)
+
     for ch_type in ch_types:
         if ch_type not in epochs:
             continue
@@ -229,6 +231,8 @@ def get_rejection_threshold(epochs, decim=1, random_state=None,
             picks = pick_types(epochs.info, meg=False, eog=True)
         elif ch_type == 'grad':
             picks = pick_types(epochs.info, meg='grad', eeg=False)
+        elif ch_type in ['hbo', 'hbr']:
+            picks = pick_types(epochs.info, meg=False, fnirs=ch_type)
 
         X = epochs.get_data()[:, picks, :]
         n_epochs, n_channels, n_times = X.shape
@@ -361,7 +365,7 @@ def _compute_thresh(this_data, method='bayesian_optimization',
 
 def compute_thresholds(epochs, method='bayesian_optimization',
                        random_state=None, picks=None, augment=True,
-                       verbose='progressbar', n_jobs=1):
+                       verbose=True, n_jobs=1):
     """Compute thresholds for each channel.
 
     Parameters
@@ -372,17 +376,20 @@ def compute_thresholds(epochs, method='bayesian_optimization',
         'bayesian_optimization' or 'random_search'
     random_state : int seed, RandomState instance, or None (default)
         The seed of the pseudo random number generator to use
-    picks : ndarray, shape(n_channels,) | None
-        The channels to be considered for autoreject. If None, defaults
-        to data channels {'meg', 'eeg'}.
+    picks : str | list | slice | None
+        Channels to include. Slices and lists of integers will be interpreted
+        as channel indices. In lists, channel *type* strings (e.g.,
+        ``['meg', 'eeg']``) will pick channels of those types, channel *name*
+        strings (e.g., ``['MEG0111', 'MEG2623']`` will pick the given channels.
+        Can also be the string values ``'all'`` to pick all channels, or
+        ``'data'`` to pick data channels. None (default) will pick data
+        channels {'meg', 'eeg'}. Note that channels in ``info['bads']`` *will
+        be included* if their names or indices are explicitly provided.
     augment : boolean
         Whether to augment the data or not. By default it is True, but
         set it to False, if the channel locations are not available.
-    verbose : 'tqdm', 'tqdm_notebook', 'progressbar' or False
+    verbose : boolean
         The verbosity of progress messages.
-        If `'progressbar'`, use `mne.utils.ProgressBar`.
-        If `'tqdm'`, use `tqdm.tqdm`.
-        If `'tqdm_notebook'`, use `tqdm.tqdm_notebook`.
         If False, suppress all output messages.
     n_jobs : int
         Number of jobs to run in parallel
@@ -396,7 +403,8 @@ def compute_thresholds(epochs, method='bayesian_optimization',
     --------
     For example, we can compute the channel-level thresholds for all the
     EEG sensors this way:
-        >>> compute_thresholds(epochs)
+
+    >>> compute_thresholds(epochs)
     """
     return _compute_thresholds(epochs, method=method,
                                random_state=random_state, picks=picks,
@@ -405,7 +413,7 @@ def compute_thresholds(epochs, method='bayesian_optimization',
 
 def _compute_thresholds(epochs, method='bayesian_optimization',
                         random_state=None, picks=None, augment=True,
-                        dots=None, verbose='progressbar', n_jobs=1):
+                        dots=None, verbose=True, n_jobs=1):
     if method not in ['bayesian_optimization', 'random_search']:
         raise ValueError('`method` param not recognized')
     picks = _handle_picks(info=epochs.info, picks=picks)
@@ -451,24 +459,29 @@ class _AutoReject(BaseAutoReject):
 
     Parameters
     ----------
-    epochs : instance of mne.Epochs
-        The epochs object
+    n_interpolate : int (default 0)
+        Number of channels for which to interpolate. This is :math:`\\rho`.
     consensus : float (0 to 1.0)
         Percentage of channels that must agree as a fraction of
         the total number of channels. This sets :math:`\\kappa/Q`.
-    n_interpolate : int (default 0)
-        Number of channels for which to interpolate. This is :math:`\\rho`.
     thresh_func : callable | None
         Function which returns the channel-level thresholds. If None,
         defaults to :func:`autoreject.compute_thresholds`.
-    picks : ndarray, shape(n_channels,) | None
-        The channels to be considered for autoreject. If None, defaults
-        to data channels {'meg', 'eeg'}.
-    verbose : 'tqdm', 'tqdm_notebook', 'progressbar' or False
+    picks : str | list | slice | None
+        Channels to include. Slices and lists of integers will be interpreted
+        as channel indices. In lists, channel *type* strings (e.g.,
+        ``['meg', 'eeg']``) will pick channels of those types, channel *name*
+        strings (e.g., ``['MEG0111', 'MEG2623']`` will pick the given channels.
+        Can also be the string values ``'all'`` to pick all channels, or
+        ``'data'`` to pick data channels. None (default) will pick data
+        channels {'meg', 'eeg'}. Note that channels in ``info['bads']`` *will
+        be included* if their names or indices are explicitly provided.
+    thresh_method : str
+        'bayesian_optimization' or 'random_search'.
+    dots : tuple
+        2-length tuple returned by utils._compute_dots.
+    verbose : boolean
         The verbosity of progress messages.
-        If `'progressbar'`, use `mne.utils.ProgressBar`.
-        If `'tqdm'`, use `tqdm.tqdm`.
-        If `'tqdm_notebook'`, use `tqdm.tqdm_notebook`.
         If False, suppress all output messages.
 
     Attributes
@@ -488,11 +501,9 @@ class _AutoReject(BaseAutoReject):
         and the peak-to-peak thresholds as the values.
     """
 
-    def __init__(self, consensus=0.1,
-                 n_interpolate=0, thresh_func=None,
-                 method='bayesian_optimization',
-                 picks=None, dots=None,
-                 verbose='progressbar'):
+    def __init__(self, n_interpolate=0, consensus=0.1, thresh_func=None,
+                 picks=None, thresh_method='bayesian_optimization',  dots=None,
+                 verbose=True):
         """Init it."""
         if thresh_func is None:
             thresh_func = _compute_thresholds
@@ -542,7 +553,7 @@ class _AutoReject(BaseAutoReject):
 
     def _get_epochs_interpolation(self, epochs, labels,
                                   picks, n_interpolate,
-                                  verbose='progressbar'):
+                                  verbose=True):
         """Interpolate the bad epochs."""
         # 1: bad segment, # 2: interpolated
         assert labels.shape[0] == len(epochs)
@@ -600,16 +611,24 @@ class _AutoReject(BaseAutoReject):
         ----------
         epochs : instance of mne.Epochs
             The epochs from which to get the drop logs.
-        picks : np.ndarray, shape(n_channels, ) | list | None
-            The channel indices to be used. If None, the .picks attribute
-            will be used.
+        picks : str | list | slice | None
+            Channels to include. Slices and lists of integers will be
+            interpreted as channel indices. In lists, channel *type* strings
+            (e.g., ``['meg', 'eeg']``) will pick channels of those types,
+            channel *name* strings (e.g., ``['MEG0111', 'MEG2623']`` will pick
+            the given channels. Can also be the string values ``'all'`` to pick
+            all channels, or ``'data'`` to pick data channels. None (default)
+            will use the .picks attribute. Note that channels in
+            ``info['bads']`` *will be included* if their names or indices are
+            explicitly provided.
 
         Returns
         -------
         reject_log : instance of autoreject.RejectLog
             The rejection log.
         """
-        picks = (self.picks_ if picks is None else picks)
+        picks = (self.picks_ if picks is None else
+                 _handle_picks(epochs.info, picks))
         picks_by_type = _get_picks_by_type(picks=picks, info=epochs.info)
         assert len(picks_by_type) == 1
         ch_type, this_picks = picks_by_type[0]
@@ -719,7 +738,7 @@ class _AutoReject(BaseAutoReject):
 
 
 def _interpolate_bad_epochs(
-        epochs, interp_channels, picks, dots=None, verbose='progressbar'):
+        epochs, interp_channels, picks, dots=None, verbose=True):
     """Actually do the interpolation."""
     assert len(epochs) == len(interp_channels)
     pos = 2
@@ -820,31 +839,34 @@ class AutoReject(object):
 
     Parameters
     ----------
+    n_interpolate : array | None
+        The values to try for the number of channels for which to interpolate.
+        This is :math:`\\rho`. If None, defaults to np.array([1, 4, 32])
     consensus : array | None
         The values to try for percentage of channels that must agree as a
         fraction of the total number of channels. This sets :math:`\\kappa/Q`.
         If None, defaults to `np.linspace(0, 1.0, 11)`
-    n_interpolate : array | None
-        The values to try for the number of channels for which to interpolate.
-        This is :math:`\\rho`. If None, defaults to np.array([1, 4, 32])
     cv : a scikit-learn cross-validation object
         Defaults to cv=10
-    picks : ndarray, shape(n_channels) | None
-        The channels to be considered for autoreject. If None, defaults
-        to data channels {'meg', 'eeg'}, which will lead fitting and combining
-        autoreject solutions across these channel types. Note that, if picks is
-        None, autoreject ignores channels marked bad in epochs.info['bads'].
+    picks : str | list | slice | None
+        Channels to include. Slices and lists of integers will be interpreted
+        as channel indices. In lists, channel *type* strings (e.g.,
+        ``['meg', 'eeg']``) will pick channels of those types, channel *name*
+        strings (e.g., ``['MEG0111', 'MEG2623']`` will pick the given channels.
+        Can also be the string values ``'all'`` to pick all channels, or
+        ``'data'`` to pick data channels. None (default) will pick data
+        channels {'meg', 'eeg'}, which will lead fitting and combining
+        autoreject solutions across these channel types. Note that channels in
+        ``info['bads']`` *will be included* if their names or indices are
+        explicitly provided.
     thresh_method : str
         'bayesian_optimization' or 'random_search'
     n_jobs : int
         The number of jobs.
     random_state : int seed, RandomState instance, or None (default)
         The seed of the pseudo random number generator to use.
-    verbose : 'tqdm', 'tqdm_notebook', 'progressbar' or False
+    verbose : boolean
         The verbosity of progress messages.
-        If `'progressbar'`, use `mne.utils.ProgressBar`.
-        If `'tqdm'`, use `tqdm.tqdm`.
-        If `'tqdm_notebook'`, use `tqdm.tqdm_notebook`.
         If False, suppress all output messages.
 
     Attributes
@@ -868,14 +890,14 @@ class AutoReject(object):
     def __init__(self, n_interpolate=None, consensus=None,
                  thresh_func=None, cv=10, picks=None,
                  thresh_method='bayesian_optimization',
-                 n_jobs=1, random_state=None, verbose='progressbar'):
+                 n_jobs=1, random_state=None, verbose=True):
         """Init it."""
         self.n_interpolate = n_interpolate
         self.consensus = consensus
         self.thresh_method = thresh_method
         self.cv = cv
         self.verbose = verbose
-        self.picks = picks  # XXX : should maybe be ch_types?
+        self.picks = picks
         self.n_jobs = n_jobs
         self.random_state = random_state
 
@@ -943,7 +965,7 @@ class AutoReject(object):
         self : instance of AutoReject
             The instance.
         """
-        self.picks_ = _handle_picks(picks=self.picks, info=epochs.info)
+        self.picks_ = _handle_picks(info=epochs.info, picks=self.picks)
         _check_data(epochs, picks=self.picks_, verbose=self.verbose)
         self.cv_ = self.cv
         if isinstance(self.cv_, int):
@@ -1021,9 +1043,16 @@ class AutoReject(object):
         ----------
         epochs : instance of mne.Epochs
             The epoched data for which the reject log is computed.
-        picks : np.ndarray, shape(n_channels, ) | list | None
-            The channel indices to be used. If None, the .picks attribute
-            will be used.
+        picks : str | list | slice | None
+            Channels to include. Slices and lists of integers will be
+            interpreted as channel indices. In lists, channel *type* strings
+            (e.g., ``['meg', 'eeg']``) will pick channels of those types,
+            channel *name* strings (e.g., ``['MEG0111', 'MEG2623']`` will pick
+            the given channels. Can also be the string values ``'all'`` to pick
+            all channels, or ``'data'`` to pick data channels. None (default)
+            will use the .picks attribute. Note that channels in
+            ``info['bads']`` *will be included* if their names or indices are
+            explicitly provided.
 
         Returns
         -------
@@ -1110,7 +1139,7 @@ class AutoReject(object):
         return self.fit(epochs).transform(epochs, return_log=return_log)
 
     def save(self, fname, overwrite=False):
-        """Save autoreject object.
+        """Save autoreject object with the HDF5 format.
 
         Parameters
         ----------
@@ -1201,42 +1230,90 @@ class RejectLog(object):
         assert len(bad_epochs) == labels.shape[0]
         assert len(ch_names) == labels.shape[1]
 
-    def plot(self, orientation='vertical', show=True):
-        """Plot.
+    def plot(self, orientation='vertical', show_names='auto', show=True,
+             ax=None):
+        """Plot an image of good, bad and interpolated channels for each epoch.
 
         Parameters
         ----------
         orientation : 'vertical' or 'horizontal'
-            If `'vertical'`, will plot sensors on x-axis and epochs on y-axis.
-            If `'horizontal'`, will plot epochs on x-axis and sensors
-            on y-axis.
+            If `'vertical'` (default), will plot sensors on x-axis and epochs
+            on y-axis. If `'horizontal'`, will plot epochs on x-axis and
+            sensors on y-axis.
+        show_names : 'auto' | int
+            If 'auto' (default), show all channel names if fewer than 25
+            entries. Otherwise it shows every 5 entries. If int, show every
+            show_names entries.
         show : bool
-            If True, display the figure immediately.
+            If True (default), display the figure immediately.
+        ax : matplotlib.axes.Axes | None
+            The axes to plot to. In ``None`` (default), create a new
+            figure and axes.
 
         Returns
         -------
         figure : Instance of matplotlib.figure.Figure
+            The figure object containing the plot.
         """
+        import matplotlib as mpl
         import matplotlib.pyplot as plt
+        import matplotlib.patches as patches
 
-        figure, ax = plt.subplots(figsize=(12, 6))
+        if show_names == 'auto':
+            show_names = 1 if len(self.ch_names) < 25 else 5
+
+        if ax is None:
+            figure, ax = plt.subplots(figsize=(12, 6))
+        else:
+            figure = ax.get_figure()
         ax.grid(False)
-        ch_names_ = self.ch_names[7::10]
+        ch_names_ = self.ch_names[::show_names]
 
+        image = self.labels.copy()
+        image[image == 2] = 0.5  # move interp to 0.5
+        # good, interp, bad
+        legend_label = {0: 'good', 0.5: 'interpolated', 1: 'bad'}
+        cmap = mpl.colors.ListedColormap(['lightgreen', 'blue', 'red'])
         if orientation == 'horizontal':
-            ax.imshow(self.labels.T, cmap='Reds',
-                      interpolation='nearest')
+            img = ax.imshow(image.T, cmap=cmap,
+                            vmin=0, vmax=1, interpolation='nearest')
             ax.set_xlabel('Epochs')
             ax.set_ylabel('Channels')
-            plt.setp(ax, yticks=range(7, self.labels.shape[1], 10),
+            plt.setp(ax, yticks=range(0, self.labels.shape[1], show_names),
                      yticklabels=ch_names_)
+            plt.setp(ax.get_yticklabels(), fontsize=8)
+            # add red box around rejected epochs
+            for idx in np.where(self.bad_epochs)[0]:
+                ax.add_patch(patches.Rectangle(
+                    (idx - 0.5, -0.5), 1, len(self.ch_names), linewidth=1,
+                    edgecolor='r', facecolor='none'))
+
+            # add legend
+            handles = [patches.Patch(color=img.cmap(img.norm(i)), label=label)
+                       for i, label in legend_label.items()]
+            ax.legend(handles=handles, bbox_to_anchor=(0.7, 1.2), ncol=3,
+                      borderaxespad=0.)
+
         elif orientation == 'vertical':
-            ax.imshow(self.labels, cmap='Reds',
-                      interpolation='nearest')
+            img = ax.imshow(image, cmap=cmap,
+                            vmin=0, vmax=1, interpolation='nearest')
             ax.set_xlabel('Channels')
             ax.set_ylabel('Epochs')
-            plt.setp(ax, xticks=range(7, self.labels.shape[1], 10),
+            plt.setp(ax, xticks=range(0, self.labels.shape[1], show_names),
                      xticklabels=ch_names_)
+            plt.setp(ax.get_xticklabels(), fontsize=8, rotation='vertical')
+            # add red box around rejected epochs
+            for idx in np.where(self.bad_epochs)[0]:
+                ax.add_patch(patches.Rectangle(
+                    (-0.5, idx - 0.5), len(self.ch_names), 1, linewidth=1,
+                    edgecolor='r', facecolor='none'))
+
+            # add legend
+            handles = [patches.Patch(color=img.cmap(img.norm(i)), label=label)
+                       for i, label in legend_label.items()]
+            ax.legend(handles=handles, bbox_to_anchor=(0.7, 1.2), ncol=3,
+                      borderaxespad=0.)
+
         else:
             msg = """orientation can be only \
                   'horizontal' or 'vertical'. Got %s""" % orientation
@@ -1246,7 +1323,7 @@ class RejectLog(object):
         plt.setp(ax.get_yticklabels(), rotation=0)
         plt.setp(ax.get_xticklabels(), rotation=90)
         ax.tick_params(axis=u'both', which=u'both', length=0)
-        plt.tight_layout(rect=[None, None, None, 1.1])
+        figure.tight_layout()
         if show:
             plt.show()
         return figure
