@@ -10,6 +10,7 @@ import numpy as np
 
 import mne
 from mne import pick_types, pick_info
+from mne.io.pick import _picks_to_idx
 from mne.channels.interpolation import _do_interp_dots
 
 
@@ -28,7 +29,7 @@ def _check_ch_locs(chs):
 
 
 def _check_data(epochs, picks, ch_constraint='data_channels',
-                verbose='progressbar'):
+                verbose=True):
     BaseEpochs = _get_epochs_type()
     if not isinstance(epochs, BaseEpochs):
         raise ValueError('Only accepts MNE epochs objects.')
@@ -48,11 +49,13 @@ def _check_data(epochs, picks, ch_constraint='data_channels',
 
     # XXX : ch_constraint -> allow_many_types=True | False
     if ch_constraint == 'data_channels':
-        if not all(ch in ('mag', 'grad', 'eeg') for ch in ch_types_picked):
+        if not all(ch in ('mag', 'grad', 'eeg', 'hbo', 'hbr')
+                   for ch in ch_types_picked):
             raise ValueError('AutoReject only supports mag, grad, and eeg '
                              'at this point.')
     elif ch_constraint == 'single_channel_type':
-        if sum(ch in ch_types_picked for ch in ('mag', 'grad', 'eeg')) > 1:
+        if sum(ch in ch_types_picked for ch in ('mag', 'grad', 'eeg',
+                                                'hbo', 'hbr')) > 1:
             raise ValueError('AutoReject only supports mag, grad, and eeg '
                              'at this point.')  # XXX: to check
     else:
@@ -69,10 +72,10 @@ def _check_data(epochs, picks, ch_constraint='data_channels',
 def _handle_picks(info, picks):
     """Pick the data channls or return picks."""
     if picks is None:
-        out = mne.pick_types(
-            info, meg=True, eeg=True, ref_meg=False, exclude='bads')
+        out = mne.pick_types(info, meg=True, eeg=True, ref_meg=False,
+                             fnirs=True, exclude='bads')
     else:
-        out = picks
+        out = _picks_to_idx(info, picks, exclude='bads')
     return out
 
 
@@ -162,33 +165,19 @@ def _pprint(params, offset=0, printer=repr):
     return lines
 
 
-def _pbar(iterable, desc, leave=True, position=None, verbose='progressbar'):
-
-    verbose = False if verbose == 0 else verbose
-    if verbose is not False and \
-            verbose not in ['progressbar', 'tqdm', 'tqdm_notebook']:
-        raise ValueError('verbose must be one of {progressbar,'
-                         'tqdm, tqdm_notebook, False}. Got %s' % verbose)
-
-    try:
-        from tqdm import tqdm
-        verbose = 'tqdm'
-    except ImportError:
-        pass
-
-    if verbose == 'progressbar':
-        from mne.utils import ProgressBar
-        pbar = ProgressBar(iterable, mesg=desc)
-    # XXX: remove the tqdm option after a few releases of MNE since it
-    # natively supported by the MNE progressbar
-    elif verbose == 'tqdm':
-        pbar = tqdm(iterable, desc=desc, leave=leave, position=position,
-                    dynamic_ncols=True)
-    elif verbose == 'tqdm_notebook':
-        from tqdm import tqdm_notebook
-        pbar = tqdm_notebook(iterable, desc=desc, leave=leave,
-                             position=position, dynamic_ncols=True)
-    elif verbose is False:
+def _pbar(iterable, desc, verbose=True, **kwargs):
+    if (isinstance(verbose, str) and
+            verbose not in {"tqdm", "tqdm_notebook", "progressbar"}):
+        raise ValueError("verbose must be a boolean value. Got %s" % verbose)
+    elif isinstance(verbose, (int, str)):
+        warnings.warn(
+            (f"verbose flag only supports boolean inputs. Option {verbose} "
+                f"coerced into type {bool(verbose)}"), DeprecationWarning)
+        verbose = bool(verbose)
+    if verbose:
+        from mne.utils.progressbar import ProgressBar
+        pbar = ProgressBar(iterable, mesg=desc, **kwargs)
+    else:
         pbar = iterable
     return pbar
 
@@ -201,21 +190,24 @@ def _get_epochs_type():
     return BaseEpochs
 
 
-def clean_by_interp(inst, picks=None, verbose='progressbar'):
+def clean_by_interp(inst, picks=None, verbose=True):
     """Clean epochs/evoked by LOOCV.
 
     Parameters
     ----------
     inst : instance of mne.Evoked or mne.Epochs
         The evoked or epochs object.
-    picks : ndarray, shape(n_channels,) | None
-        The channels to be considered for autoreject. If None, defaults
-        to data channels {'meg', 'eeg'}.
-    verbose : 'tqdm', 'tqdm_notebook', 'progressbar' or False
+    picks : str | list | slice | None
+        Channels to include. Slices and lists of integers will be interpreted
+        as channel indices. In lists, channel *type* strings (e.g.,
+        ``['meg', 'eeg']``) will pick channels of those types, channel *name*
+        strings (e.g., ``['MEG0111', 'MEG2623']`` will pick the given channels.
+        Can also be the string values ``'all'`` to pick all channels, or
+        ``'data'`` to pick data channels. None (default) will pick data
+        channels {'meg', 'eeg'}. Note that channels in ``info['bads']`` *will
+        be included* if their names or indices are explicitly provided.
+    verbose : boolean
         The verbosity of progress messages.
-        If `'progressbar'`, use `mne.utils.ProgressBar`.
-        If `'tqdm'`, use `tqdm.tqdm`.
-        If `'tqdm_notebook'`, use `tqdm.tqdm_notebook`.
         If False, suppress all output messages.
 
     Returns
@@ -226,7 +218,7 @@ def clean_by_interp(inst, picks=None, verbose='progressbar'):
     return _clean_by_interp(inst, picks=picks, verbose=verbose)
 
 
-def _clean_by_interp(inst, picks=None, dots=None, verbose='progressbar'):
+def _clean_by_interp(inst, picks=None, dots=None, verbose=True):
     inst_interp = inst.copy()
     mesg = 'Creating augmented epochs'
     picks = _handle_picks(info=inst_interp.info, picks=picks)
@@ -249,28 +241,10 @@ def _clean_by_interp(inst, picks=None, dots=None, verbose='progressbar'):
         else:
             raise ValueError('Unrecognized type for inst')
         inst._data[:, pick_interp] = data_orig.copy()
+    inst.info['bads'] = inst_interp.info['bads'].copy()
     return inst_interp
 
 
-def fetch_file(url, file_name, resume=True, timeout=10.):
-    """Load requested file, downloading it if needed or requested.
-
-    Parameters
-    ----------
-    url: string
-        The url of file to be downloaded.
-    file_name: string
-        Name, along with the path, of where downloaded file will be saved.
-    resume: bool, optional
-        If true, try to resume partially downloaded files.
-    timeout : float
-        The URL open timeout.
-    """
-    from mne.utils import _fetch_file
-    _fetch_file(url=url, file_name=file_name, print_destination=True,
-                resume=resume, hash_=None, timeout=timeout)
-
-
 def interpolate_bads(inst, picks, dots=None, reset_bads=True, mode='accurate'):
     """Interpolate bad MEG and EEG channels."""
     import mne
@@ -298,7 +272,7 @@ def interpolate_bads(inst, picks, dots=None, reset_bads=True, mode='accurate'):
     return inst
 
 
-def _interpolate_bads_eeg(inst, picks=None, verbose=None):
+def _interpolate_bads_eeg(inst, picks=None, verbose=False):
     """ Interpolate bad EEG channels.
 
     Operates in place.
@@ -307,8 +281,13 @@ def _interpolate_bads_eeg(inst, picks=None, verbose=None):
     ----------
     inst : mne.io.Raw, mne.Epochs or mne.Evoked
         The data to interpolate. Must be preloaded.
-    picks: np.ndarray, shape(n_channels, ) | list | None
-        The channel indices to be used for interpolation.
+    picks : str | list | slice | None
+        Channels to include for interpolation. Slices and lists of integers
+        will be interpreted as channel indices. In lists, channel *name*
+        strings (e.g., ``['EEG 01', 'EEG 02']``) will pick the given channels.
+        None (default) will pick all EEG channels. Note that channels in
+        ``info['bads']`` *will be included* if their names or indices are
+        explicitly provided.
     """
     from mne.bem import _fit_sphere
     from mne.utils import logger, warn
@@ -316,13 +295,14 @@ def _interpolate_bads_eeg(inst, picks=None, verbose=None):
     from mne.channels.interpolation import _make_interpolation_matrix
     import numpy as np
 
+    inst.info._check_consistency()
     if picks is None:
         picks = pick_types(inst.info, meg=False, eeg=True, exclude=[])
+    else:
+        picks = _handle_picks(inst.info, picks)
 
     bads_idx = np.zeros(len(inst.ch_names), dtype=np.bool)
     goods_idx = np.zeros(len(inst.ch_names), dtype=np.bool)
-
-    inst.info._check_consistency()
     bads_idx[picks] = [inst.ch_names[ch] in inst.info['bads'] for ch in picks]
 
     if len(picks) == 0 or bads_idx.sum() == 0:
@@ -357,7 +337,7 @@ def _interpolate_bads_eeg(inst, picks=None, verbose=None):
 
 
 def _interpolate_bads_meg_fast(inst, picks, mode='accurate',
-                               dots=None, verbose=None):
+                               dots=None, verbose=False):
     """Interpolate bad channels from data in good channels."""
     # We can have pre-picked instances or not.
     # And we need to handle it.
@@ -414,7 +394,6 @@ def _interpolate_bads_meg_fast(inst, picks, mode='accurate',
 
 def _compute_dots(info, mode='fast'):
     """Compute all-to-all dots."""
-    from mne.forward._field_interpolation import _setup_dots
     from mne.forward._lead_dots import _do_self_dots, _do_cross_dots
     from mne.forward._make_forward import _create_meg_coils, _read_coil_defs
     from mne.bem import _check_origin
@@ -423,7 +402,8 @@ def _compute_dots(info, mode='fast'):
     coils = _create_meg_coils(info['chs'], 'normal', info['dev_head_t'],
                               templates)
     my_origin = _check_origin((0., 0., 0.04), info)
-    int_rad, noise, lut_fun, n_fact = _setup_dots(mode, coils, 'meg')
+    int_rad, noise, lut_fun, n_fact = _patch_setup_dots(mode, info,
+                                                        coils, 'meg')
     self_dots = _do_self_dots(int_rad, False, coils, my_origin, 'meg',
                               lut_fun, n_fact, n_jobs=1)
     cross_dots = _do_cross_dots(int_rad, False, coils, coils,
@@ -443,7 +423,6 @@ def _pick_dots(dots, pick_from, pick_to):
 def _fast_map_meg_channels(info, pick_from, pick_to,
                            dots=None, mode='fast'):
     from mne.io.pick import pick_info
-    from mne.forward._field_interpolation import _setup_dots
     from mne.forward._field_interpolation import _compute_mapping_matrix
     from mne.forward._make_forward import _create_meg_coils, _read_coil_defs
     from mne.bem import _check_origin
@@ -459,7 +438,8 @@ def _fast_map_meg_channels(info, pick_from, pick_to,
     coils_from = _create_meg_coils(info_from['chs'], 'normal',
                                    info_from['dev_head_t'], templates)
     my_origin = _check_origin((0., 0., 0.04), info_from)
-    int_rad, noise, lut_fun, n_fact = _setup_dots(mode, coils_from, 'meg')
+    int_rad, noise, lut_fun, n_fact = _patch_setup_dots(mode, info_from,
+                                                        coils_from, 'meg')
 
     # This function needs a clean input. It hates the presence of other
     # channels than MEG channels. Make sure all is picked.
@@ -479,3 +459,13 @@ def _fast_map_meg_channels(info, pick_from, pick_to,
     mne.set_log_level(verbose)
 
     return fmd['data']
+
+
+def _patch_setup_dots(mode, info, coils, ch):
+    """Monkey patch _setup_dots for MNE-Python >= v0.24."""
+    from mne.forward._field_interpolation import _setup_dots
+    from mne.utils import check_version
+    if not check_version('mne', '0.24'):
+        return _setup_dots(mode, coils, ch)
+    else:
+        return _setup_dots(mode, info, coils, ch)
